"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MessageSelectionError = exports.MessageResolutionError = exports.MessageDataModelError = exports.MissingSyntaxError = exports.MessageSyntaxError = exports.MessageError = void 0;
const from_cst_js_1 = require("./data-model/from-cst.js");
/**
 * Base error class used by MessageFormat
 *
 * @beta
 */
class MessageError extends Error {
    constructor(type, message) {
        super(message);
        this.type = type;
    }
}
exports.MessageError = MessageError;
/**
 * Errors in the message syntax.
 *
 * @beta
 */
class MessageSyntaxError extends MessageError {
    constructor(type, start, end, expected) {
        let message;
        switch (type) {
            case 'empty-token':
            case 'bad-escape':
            case 'bad-input-expression':
            case 'bad-selector':
            case 'extra-content':
            case 'parse-error':
                message = `Syntax parse error: ${type} at ${start}`;
                break;
            case 'missing-syntax':
                message = `Syntax parse error: Missing ${expected} at ${start}`;
                break;
            case 'duplicate-declaration':
            case 'duplicate-option':
            case 'forward-reference':
            case 'key-mismatch':
            case 'missing-fallback':
            case 'missing-selector-annotation':
                message = `Data model error: ${type}`;
                if (start >= 0)
                    message += ` at ${start}`;
        }
        super(type, message);
        this.start = start;
        this.end = end;
    }
}
exports.MessageSyntaxError = MessageSyntaxError;
/** @beta */
class MissingSyntaxError extends MessageSyntaxError {
    constructor(pos, expected) {
        super('missing-syntax', pos, pos + expected.length, expected);
        this.expected = expected;
    }
}
exports.MissingSyntaxError = MissingSyntaxError;
/**
 * Errors in the message data model.
 *
 * @beta
 */
class MessageDataModelError extends MessageSyntaxError {
    constructor(type, node) {
        const { start, end } = node[from_cst_js_1.cst] ?? { start: -1, end: -1 };
        super(type, start, end);
    }
}
exports.MessageDataModelError = MessageDataModelError;
/**
 * Message runtime resolution errors
 *
 * @beta
 */
class MessageResolutionError extends MessageError {
    constructor(type, message, source) {
        super(type, message);
        this.source = source;
    }
}
exports.MessageResolutionError = MessageResolutionError;
/**
 * Errors in message selection.
 *
 * @beta
 */
class MessageSelectionError extends MessageError {
    constructor(type) {
        super(type, `Selection error: ${type}`);
    }
}
exports.MessageSelectionError = MessageSelectionError;
