"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.datetime = void 0;
const errors_js_1 = require("../errors.js");
const utils_js_1 = require("./utils.js");
/**
 * `datetime` accepts an optional Date, number or string as its input
 * and formats it with the same options as
 * {@link https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Intl/DateTimeFormat | Intl.DateTimeFormat}.
 * If not given any argument, the current date/time is used.
 *
 * @beta
 */
function datetime({ localeMatcher, locales, source }, options, input) {
    const lc = (0, utils_js_1.mergeLocales)(locales, input, options);
    const opt = { localeMatcher };
    if (input && typeof input === 'object') {
        if (input && 'options' in input)
            Object.assign(opt, input.options);
        if (!(input instanceof Date) && typeof input.valueOf === 'function') {
            input = input.valueOf();
        }
    }
    let value;
    switch (typeof input) {
        case 'number':
        case 'string':
            value = new Date(input);
            break;
        case 'object':
            value = input;
            break;
        case 'undefined':
            value = new Date();
            break;
    }
    if (!(value instanceof Date)) {
        const msg = 'Input is not a date';
        throw new errors_js_1.MessageResolutionError('bad-input', msg, source);
    }
    for (const [name, value] of Object.entries(options)) {
        if (value === undefined)
            continue;
        try {
            switch (name) {
                case 'locale':
                    break;
                case 'fractionalSecondDigits':
                    // @ts-expect-error TS types don't know about fractionalSecondDigits
                    opt[name] = (0, utils_js_1.asPositiveInteger)(value);
                    break;
                case 'hour12':
                    opt[name] = (0, utils_js_1.asBoolean)(value);
                    break;
                default:
                    // @ts-expect-error Unknown options will be ignored
                    opt[name] = (0, utils_js_1.asString)(value);
            }
        }
        catch {
            const msg = `Value ${value} is not valid for :datetime option ${name}`;
            throw new errors_js_1.MessageResolutionError('bad-option', msg, source);
        }
    }
    const date = value;
    let locale;
    let dtf;
    let str;
    return {
        type: 'datetime',
        source,
        get locale() {
            return (locale ?? (locale = Intl.DateTimeFormat.supportedLocalesOf(lc, opt)[0]));
        },
        get options() {
            return { ...opt };
        },
        toParts() {
            dtf ?? (dtf = new Intl.DateTimeFormat(lc, options));
            const parts = dtf.formatToParts(date);
            locale ?? (locale = dtf.resolvedOptions().locale);
            return [{ type: 'datetime', source, locale, parts }];
        },
        toString() {
            dtf ?? (dtf = new Intl.DateTimeFormat(lc, options));
            str ?? (str = dtf.format(date));
            return str;
        },
        valueOf: () => date
    };
}
exports.datetime = datetime;
