"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.plural = exports.ordinal = exports.integer = exports.number = void 0;
const errors_js_1 = require("../errors.js");
const utils_js_1 = require("./utils.js");
const NotFormattable = Symbol('not-formattable');
/**
 * `number` accepts a number, BigInt or string representing a JSON number as input
 * and formats it with the same options as
 * {@link https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Intl/NumberFormat | Intl.NumberFormat}.
 * It also supports plural category selection via
 * {@link https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Intl/PluralRules | Intl.PluralRules}.
 *
 * @beta
 */
function number({ localeMatcher, locales, source }, options, input) {
    let value;
    const opt = {
        localeMatcher
    };
    switch (typeof input) {
        case 'bigint':
        case 'number':
            value = input;
            break;
        case 'object':
            if (typeof input?.valueOf === 'function') {
                value = input.valueOf();
                if ('options' in input)
                    Object.assign(opt, input.options);
            }
            break;
        case 'string':
            try {
                value = JSON.parse(input);
            }
            catch {
                // handled below
            }
    }
    if (typeof value !== 'bigint' && typeof value !== 'number') {
        const msg = 'Input is not numeric';
        throw new errors_js_1.MessageResolutionError('bad-input', msg, source);
    }
    if (options) {
        for (const [name, value] of Object.entries(options)) {
            if (value === undefined)
                continue;
            try {
                switch (name) {
                    case 'locale':
                    case 'type': // used internally by Intl.PluralRules, but called 'select' here
                        break;
                    case 'minimumIntegerDigits':
                    case 'minimumFractionDigits':
                    case 'maximumFractionDigits':
                    case 'minimumSignificantDigits':
                    case 'maximumSignificantDigits':
                    case 'roundingIncrement':
                        // @ts-expect-error TS types don't know about roundingIncrement
                        opt[name] = (0, utils_js_1.asPositiveInteger)(value);
                        break;
                    case 'useGrouping':
                        opt[name] = (0, utils_js_1.asBoolean)(value);
                        break;
                    default:
                        // @ts-expect-error Unknown options will be ignored
                        opt[name] = (0, utils_js_1.asString)(value);
                }
            }
            catch {
                const msg = `Value ${value} is not valid for :number option ${name}`;
                throw new errors_js_1.MessageResolutionError('bad-option', msg, source);
            }
        }
    }
    const formattable = !options[NotFormattable];
    const lc = (0, utils_js_1.mergeLocales)(locales, input, options);
    const num = value;
    let locale;
    let nf;
    let cat;
    let str;
    return {
        type: 'number',
        source,
        get locale() {
            return (locale ?? (locale = Intl.NumberFormat.supportedLocalesOf(lc, opt)[0]));
        },
        get options() {
            return { ...opt };
        },
        selectKey(keys) {
            const str = String(num);
            if (keys.has(str))
                return str;
            if (opt.select === 'exact')
                return null;
            const pluralOpt = opt.select
                ? { ...opt, select: undefined, type: opt.select }
                : opt;
            // Intl.PluralRules needs a number, not bigint
            cat ?? (cat = new Intl.PluralRules(lc, pluralOpt).select(Number(num)));
            return keys.has(cat) ? cat : null;
        },
        toParts: formattable
            ? () => {
                nf ?? (nf = new Intl.NumberFormat(lc, opt));
                const parts = nf.formatToParts(num);
                locale ?? (locale = nf.resolvedOptions().locale);
                return [{ type: 'number', source, locale, parts }];
            }
            : undefined,
        toString: formattable
            ? () => {
                nf ?? (nf = new Intl.NumberFormat(lc, opt));
                str ?? (str = nf.format(num));
                return str;
            }
            : undefined,
        valueOf: () => num
    };
}
exports.number = number;
/**
 * `integer` accepts a number, BigInt or string representing a JSON number as input
 * and formats it with the same options as
 * {@link https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Intl/NumberFormat | Intl.NumberFormat}.
 * It also supports plural category selection via
 * {@link https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Intl/PluralRules | Intl.PluralRules}.
 *
 * The `maximumFractionDigits=0` and `style='decimal'` options are fixed for `:integer`.
 *
 * @beta
 */
const integer = (ctx, options, input) => number(ctx, { ...options, maximumFractionDigits: 0, style: 'decimal' }, input);
exports.integer = integer;
/**
 * `ordinal` accepts a number, BigInt or string representing a JSON number as input
 * and acts as a ordinal category selector on the value, using
 * {@link https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Intl/PluralRules | Intl.PluralRules}.
 *
 * `ordinal` cannot be used as a formatter.
 *
 * @beta
 */
const ordinal = (ctx, options, input) => number(ctx, { ...options, select: 'ordinal', [NotFormattable]: true }, input);
exports.ordinal = ordinal;
/**
 * `plural` accepts a number, BigInt or string representing a JSON number as input
 * and acts as a cardinal plural selector on the value, using
 * {@link https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Intl/PluralRules | Intl.PluralRules}.
 *
 * `plural` cannot be used as a formatter.
 *
 * @beta
 */
const plural = (ctx, options, input) => number(ctx, { ...options, select: 'cardinal', [NotFormattable]: true }, input);
exports.plural = plural;
